﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;

namespace WpfApplication2
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            DataContext = new ViewModel();
        }       
    }

    public class ViewModel
    {
        public ViewModel()
        {
            Data = new ObservableCollection<Data>()
            {
                new Data { ID = 1, Combo = 3 },
                new Data { ID = 2, Combo = 1 },
                new Data { ID = 3, Combo = 2 },
                new Data { ID = 4, Combo = 0 },
                new Data { ID = 5, Combo = 1 },
            };
        }

        public ObservableCollection<Data> Data { get; set; }

        public ICommand CellUpdatingCommnad
        {
            get
            {
                return new RelayCommand<CellUpdatingEventArgs>(new Action<CellUpdatingEventArgs>((e) =>
                    {
                        MessageBox.Show("Cell Updating!!!");
                    }));
            }
        }

        public ICommand CellUpdatedCommnad
        {
            get
            {
                return new RelayCommand<CellUpdatedEventArgs>(new Action<CellUpdatedEventArgs>((e) =>
                {
                    MessageBox.Show("Cell Updated!!!");
                }));
            }
        }
    }

    public class Data
    {
        public int ID { get; set; }
        int i;
        public int Combo { get { return i; } set { i = value; } }
    }

    public static class XDGBehaviors
    {
        public static readonly DependencyProperty CellUpdatingCommnadBehavior = EventBehaviourFactory.CreateCommandExecutionEventBehaviour(
            XamDataGrid.CellUpdatingEvent,
            "CellUpdatingCommnadBehavior",
            typeof(XDGBehaviors));

        public static void SetCellUpdatingCommnadBehavior(DependencyObject o, ICommand value)
        {
            o.SetValue(CellUpdatingCommnadBehavior, value);
        }

        public static ICommand GetCellUpdatingCommnadBehavior(DependencyObject o)
        {
            return o.GetValue(CellUpdatingCommnadBehavior) as ICommand;
        }

        public static readonly DependencyProperty CellUpdatedCommnadBehavior = EventBehaviourFactory.CreateCommandExecutionEventBehaviour(
            XamDataGrid.CellUpdatedEvent,
            "CellUpdatedCommnadBehavior",
            typeof(XDGBehaviors));

        public static void SetCellUpdatedCommnadBehavior(DependencyObject o, ICommand value)
        {
            o.SetValue(CellUpdatedCommnadBehavior, value);
        }

        public static ICommand GetCellUpdatedCommnadBehavior(DependencyObject o)
        {
            return o.GetValue(CellUpdatedCommnadBehavior) as ICommand;
        }
    }

    public static class EventBehaviourFactory
    {
        public static DependencyProperty CreateCommandExecutionEventBehaviour(RoutedEvent routedEvent, string propertyName, Type ownerType)
        {
            DependencyProperty property = DependencyProperty.RegisterAttached(
                propertyName,
                typeof(ICommand),
                ownerType,
                new PropertyMetadata(null,
                    new ExecuteCommandOnRoutedEventBehaviour(routedEvent).PropertyChangedHandler));

            return property;
        }

        private class ExecuteCommandOnRoutedEventBehaviour : ExecuteCommandBehaviour
        {
            private readonly RoutedEvent _routedEvent;

            public ExecuteCommandOnRoutedEventBehaviour(RoutedEvent routedEvent)
            {
                _routedEvent = routedEvent;
            }

            protected override void AdjustEventHandlers(DependencyObject sender, object oldValue, object newValue)
            {
                UIElement element = sender as UIElement;
                if (element == null) { return; }

                if (oldValue != null)
                {
                    element.RemoveHandler(_routedEvent, new RoutedEventHandler(EventHandler));
                }

                if (newValue != null)
                {
                    element.AddHandler(_routedEvent, new RoutedEventHandler(EventHandler));
                }
            }

            protected void EventHandler(object sender, RoutedEventArgs e)
            {
                HandleEvent(sender, e);
            }
        }

        internal abstract class ExecuteCommandBehaviour
        {
            protected DependencyProperty _property;
            protected abstract void AdjustEventHandlers(DependencyObject sender, object oldValue, object newValue);

            protected void HandleEvent(object sender, EventArgs e)
            {
                DependencyObject dp = sender as DependencyObject;
                if (dp == null)
                {
                    return;
                }

                ICommand command = dp.GetValue(_property) as ICommand;

                if (command == null)
                {
                    return;
                }

                if (command.CanExecute(e))
                {
                    command.Execute(e);
                }
            }

            /// <summary>
            /// Listens for a change in the DependencyProperty that we are assigned to, and
            /// adjusts the EventHandlers accordingly
            /// </summary>
            /// <param name="sender"></param>
            /// <param name="e"></param>
            public void PropertyChangedHandler(DependencyObject sender, DependencyPropertyChangedEventArgs e)
            {
                // the first time the property changes,
                // make a note of which property we are supposed
                // to be watching
                if (_property == null)
                {
                    _property = e.Property;
                }

                object oldValue = e.OldValue;
                object newValue = e.NewValue;

                AdjustEventHandlers(sender, oldValue, newValue);
            }
        }
    }

    public class RelayCommand<T> : ICommand
    {

        #region Declarations

        readonly Predicate<T> _canExecute;
        readonly Action<T> _execute;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class and the command can always be executed.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        public RelayCommand(Action<T> execute)
            : this(execute, null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        /// <param name="canExecute">The execution status logic.</param>
        public RelayCommand(Action<T> execute, Predicate<T> canExecute)
        {

            if (execute == null)
                throw new ArgumentNullException("execute");
            _execute = execute;
            _canExecute = canExecute;
        }

        #endregion

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add
            {

                if (_canExecute != null)
                    CommandManager.RequerySuggested += value;
            }
            remove
            {

                if (_canExecute != null)
                    CommandManager.RequerySuggested -= value;
            }
        }

        [DebuggerStepThrough]
        public virtual Boolean CanExecute(Object parameter)
        {
            return _canExecute == null ? true : _canExecute((T)parameter);
        }

        public virtual void Execute(Object parameter)
        {
            _execute((T)parameter);
        }

        #endregion
    }
}


